@@ -29,10 +29,10 @@
  */
 
 #include "avcodec.h"
-#include "bytestream.h"
 #include "get_bits.h"
 #include "dsputil.h"
 #include "aandcttab.h"
+#include "eaidct.h"
 #include "mpeg12.h"
 #include "mpeg12data.h"
 #include "libavutil/imgutils.h"
@@ -43,31 +43,28 @@
 #define MADe_TAG MKTAG('M', 'A', 'D', 'e')    /* MAD lqp-frame */
 
 typedef struct MadContext {
-    MpegEncContext s;
+    AVCodecContext *avctx;
+    DSPContext dsp;
     AVFrame frame;
     AVFrame last_frame;
+    GetBitContext gb;
     void *bitstream_buf;
     unsigned int bitstream_buf_size;
     DECLARE_ALIGNED(16, DCTELEM, block)[64];
+    ScanTable scantable;
+    uint16_t quant_matrix[64];
+    int mb_x;
+    int mb_y;
 } MadContext;
 
-static void bswap16_buf(uint16_t *dst, const uint16_t *src, int count)
-{
-    int i;
-    for (i=0; i<count; i++)
-        dst[i] = av_bswap16(src[i]);
-}
-
 static av_cold int decode_init(AVCodecContext *avctx)
 {
-    MadContext *t = avctx->priv_data;
-    MpegEncContext *s = &t->s;
+    MadContext *s = avctx->priv_data;
     s->avctx = avctx;
     avctx->pix_fmt = PIX_FMT_YUV420P;
-    if (avctx->idct_algo == FF_IDCT_AUTO)
-        avctx->idct_algo = FF_IDCT_EA;
-    dsputil_init(&s->dsp, avctx);
-    ff_init_scantable(s->dsp.idct_permutation, &s->intra_scantable, ff_zigzag_direct);
+    ff_dsputil_init(&s->dsp, avctx);
+    ff_init_scantable_permutation(s->dsp.idct_permutation, FF_NO_IDCT_PERM);
+    ff_init_scantable(s->dsp.idct_permutation, &s->scantable, ff_zigzag_direct);
     ff_mpeg12_init_vlcs();
     return 0;
 }
@@ -84,19 +81,18 @@ static inline void comp(unsigned char *dst, int dst_stride,
 static inline void comp_block(MadContext *t, int mb_x, int mb_y,
                               int j, int mv_x, int mv_y, int add)
 {
-    MpegEncContext *s = &t->s;
     if (j < 4) {
         unsigned offset = (mb_y*16 + ((j&2)<<2) + mv_y)*t->last_frame.linesize[0] + mb_x*16 + ((j&1)<<3) + mv_x;
-        if (offset >= (s->height - 7) * t->last_frame.linesize[0] - 7)
+        if (offset >= (t->avctx->height - 7) * t->last_frame.linesize[0] - 7)
             return;
         comp(t->frame.data[0] + (mb_y*16 + ((j&2)<<2))*t->frame.linesize[0] + mb_x*16 + ((j&1)<<3),
              t->frame.linesize[0],
              t->last_frame.data[0] + offset,
              t->last_frame.linesize[0], add);
-    } else if (!(s->avctx->flags & CODEC_FLAG_GRAY)) {
+    } else if (!(t->avctx->flags & CODEC_FLAG_GRAY)) {
         int index = j - 3;
         unsigned offset = (mb_y * 8 + (mv_y/2))*t->last_frame.linesize[index] + mb_x * 8 + (mv_x/2);
-        if (offset >= (s->height/2 - 7) * t->last_frame.linesize[index] - 7)
+        if (offset >= (t->avctx->height/2 - 7) * t->last_frame.linesize[index] - 7)
             return;
         comp(t->frame.data[index] + (mb_y*8)*t->frame.linesize[index] + mb_x * 8,
              t->frame.linesize[index],
@@ -107,26 +103,24 @@ static inline void comp_block(MadContext *t, int mb_x, int mb_y,
 
 static inline void idct_put(MadContext *t, DCTELEM *block, int mb_x, int mb_y, int j)
 {
-    MpegEncContext *s = &t->s;
     if (j < 4) {
-        s->dsp.idct_put(
+        ff_ea_idct_put_c(
             t->frame.data[0] + (mb_y*16 + ((j&2)<<2))*t->frame.linesize[0] + mb_x*16 + ((j&1)<<3),
             t->frame.linesize[0], block);
-    } else if (!(s->avctx->flags & CODEC_FLAG_GRAY)) {
+    } else if (!(t->avctx->flags & CODEC_FLAG_GRAY)) {
         int index = j - 3;
-        s->dsp.idct_put(
+        ff_ea_idct_put_c(
             t->frame.data[index] + (mb_y*8)*t->frame.linesize[index] + mb_x*8,
             t->frame.linesize[index], block);
     }
 }
 
-static inline int decode_block_intra(MadContext * t, DCTELEM * block)
+static inline int decode_block_intra(MadContext *s, DCTELEM * block)
 {
-    MpegEncContext *s = &t->s;
     int level, i, j, run;
     RLTable *rl = &ff_rl_mpeg1;
-    const uint8_t *scantable = s->intra_scantable.permutated;
-    int16_t *quant_matrix = s->intra_matrix;
+    const uint8_t *scantable = s->scantable.permutated;
+    int16_t *quant_matrix = s->quant_matrix;
 
     block[0] = (128 + get_sbits(&s->gb, 8)) * quant_matrix[0];
 
@@ -144,11 +138,6 @@ static inline int decode_block_intra(MadContext * t, DCTELEM * block)
                 break;
             } else if (level != 0) {
                 i += run;
-                if (i > 63) {
-                    av_log(s->avctx, AV_LOG_ERROR,
-                           "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
-                    return -1;
-                }
                 j = scantable[i];
                 level = (level*quant_matrix[j]) >> 4;
                 level = (level-1)|1;
@@ -163,11 +152,6 @@ static inline int decode_block_intra(MadContext * t, DCTELEM * block)
                 run = SHOW_UBITS(re, &s->gb, 6)+1; LAST_SKIP_BITS(re, &s->gb, 6);
 
                 i += run;
-                if (i > 63) {
-                    av_log(s->avctx, AV_LOG_ERROR,
-                           "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
-                    return -1;
-                }
                 j = scantable[i];
                 if (level < 0) {
                     level = -level;
@@ -179,6 +163,10 @@ static inline int decode_block_intra(MadContext * t, DCTELEM * block)
                     level = (level-1)|1;
                 }
             }
+            if (i > 63) {
+                av_log(s->avctx, AV_LOG_ERROR, "ac-tex damaged at %d %d\n", s->mb_x, s->mb_y);
+                return -1;
+            }
 
             block[j] = level;
         }
@@ -198,9 +186,8 @@ static int decode_motion(GetBitContext *gb)
     return value;
 }
 
-static int decode_mb(MadContext *t, int inter)
+static int decode_mb(MadContext *s, int inter)
 {
-    MpegEncContext *s = &t->s;
     int mv_map = 0;
     int mv_x, mv_y;
     int j;
@@ -211,40 +198,31 @@ static int decode_mb(MadContext *t, int inter)
             mv_map = v ? get_bits(&s->gb, 6) : 63;
             mv_x = decode_motion(&s->gb);
             mv_y = decode_motion(&s->gb);
-        } else {
-            mv_map = 0;
         }
     }
 
     for (j=0; j<6; j++) {
         if (mv_map & (1<<j)) {  // mv_x and mv_y are guarded by mv_map
             int add = 2*decode_motion(&s->gb);
-            if (t->last_frame.data[0])
-                comp_block(t, s->mb_x, s->mb_y, j, mv_x, mv_y, add);
+            if (s->last_frame.data[0])
+                comp_block(s, s->mb_x, s->mb_y, j, mv_x, mv_y, add);
         } else {
-            s->dsp.clear_block(t->block);
-            if(decode_block_intra(t, t->block) < 0)
+            s->dsp.clear_block(s->block);
+            if(decode_block_intra(s, s->block) < 0)
                 return -1;
-            idct_put(t, t->block, s->mb_x, s->mb_y, j);
+            idct_put(s, s->block, s->mb_x, s->mb_y, j);
         }
     }
     return 0;
 }
 
-static void calc_intra_matrix(MadContext *t, int qscale)
+static void calc_quant_matrix(MadContext *s, int qscale)
 {
-    MpegEncContext *s = &t->s;
     int i;
 
-    if (s->avctx->idct_algo == FF_IDCT_EA) {
-        s->intra_matrix[0] = (ff_inv_aanscales[0]*ff_mpeg1_default_intra_matrix[0]) >> 11;
-        for (i=1; i<64; i++)
-            s->intra_matrix[i] = (ff_inv_aanscales[i]*ff_mpeg1_default_intra_matrix[i]*qscale + 32) >> 10;
-    } else {
-        s->intra_matrix[0] = ff_mpeg1_default_intra_matrix[0];
-        for (i=1; i<64; i++)
-            s->intra_matrix[i] = (ff_mpeg1_default_intra_matrix[i]*qscale) << 1;
-    }
+    s->quant_matrix[0] = (ff_inv_aanscales[0]*ff_mpeg1_default_intra_matrix[0]) >> 11;
+    for (i=1; i<64; i++)
+        s->quant_matrix[i] = (ff_inv_aanscales[i]*ff_mpeg1_default_intra_matrix[i]*qscale + 32) >> 10;
 }
 
 static int decode_frame(AVCodecContext *avctx,
@@ -253,70 +231,67 @@ static int decode_frame(AVCodecContext *avctx,
 {
     const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
-    MadContext *t     = avctx->priv_data;
-    GetByteContext gb;
-    MpegEncContext *s = &t->s;
+    const uint8_t *buf_end = buf+buf_size;
+    MadContext *s     = avctx->priv_data;
+    int width, height;
     int chunk_type;
     int inter;
 
-    bytestream2_init(&gb, buf, buf_size);
+    if (buf_size < 17) {
+        av_log(avctx, AV_LOG_ERROR, "Input buffer too small\n");
+        *data_size = 0;
+        return -1;
+    }
 
-    chunk_type = bytestream2_get_le32(&gb);
+    chunk_type = AV_RL32(&buf[0]);
     inter = (chunk_type == MADm_TAG || chunk_type == MADe_TAG);
-    bytestream2_skip(&gb, 10);
+    buf += 8;
 
     av_reduce(&avctx->time_base.num, &avctx->time_base.den,
-              bytestream2_get_le16(&gb), 1000, 1<<30);
+              AV_RL16(&buf[6]), 1000, 1<<30);
 
-    s->width  = bytestream2_get_le16(&gb);
-    s->height = bytestream2_get_le16(&gb);
-    bytestream2_skip(&gb, 1);
-    calc_intra_matrix(t, bytestream2_get_byte(&gb));
-    bytestream2_skip(&gb, 2);
+    width  = AV_RL16(&buf[8]);
+    height = AV_RL16(&buf[10]);
+    calc_quant_matrix(s, buf[13]);
+    buf += 16;
 
-    if (bytestream2_get_bytes_left(&gb) < 2) {
-        av_log(avctx, AV_LOG_ERROR, "Input data too small\n");
-        return AVERROR_INVALIDDATA;
+    if (avctx->width != width || avctx->height != height) {
+        if((width * height)/2048*7 > buf_end-buf)
+            return -1;
+        if (av_image_check_size(width, height, 0, avctx) < 0)
+            return -1;
+        avcodec_set_dimensions(avctx, width, height);
+        if (s->frame.data[0])
+            avctx->release_buffer(avctx, &s->frame);
+        if (s->last_frame.data[0])
+            avctx->release_buffer(avctx, &s->last_frame);
     }
 
-    if (avctx->width != s->width || avctx->height != s->height) {
-        if((s->width * s->height)/2048*7 > bytestream2_get_bytes_left(&gb))
-            return -1;
-        if (av_image_check_size(s->width, s->height, 0, avctx) < 0)
-            return -1;
-        avcodec_set_dimensions(avctx, s->width, s->height);
-        if (t->frame.data[0])
-            avctx->release_buffer(avctx, &t->frame);
-        if (t->last_frame.data[0])
-            avctx->release_buffer(avctx, &t->last_frame);
-    }
-
-    t->frame.reference = 3;
-    if (!t->frame.data[0]) {
-        if (avctx->get_buffer(avctx, &t->frame) < 0) {
+    s->frame.reference = 3;
+    if (!s->frame.data[0]) {
+        if (avctx->get_buffer(avctx, &s->frame) < 0) {
             av_log(avctx, AV_LOG_ERROR, "get_buffer() failed\n");
             return -1;
         }
     }
 
-    av_fast_malloc(&t->bitstream_buf, &t->bitstream_buf_size,
-                   bytestream2_get_bytes_left(&gb) + FF_INPUT_BUFFER_PADDING_SIZE);
-    if (!t->bitstream_buf)
+    av_fast_malloc(&s->bitstream_buf, &s->bitstream_buf_size, (buf_end-buf) + FF_INPUT_BUFFER_PADDING_SIZE);
+    if (!s->bitstream_buf)
         return AVERROR(ENOMEM);
-    bswap16_buf(t->bitstream_buf, (const uint16_t *)(buf + bytestream2_tell(&gb)),
-                bytestream2_get_bytes_left(&gb) / 2);
-    memset((uint8_t*)t->bitstream_buf + bytestream2_get_bytes_left(&gb), 0, FF_INPUT_BUFFER_PADDING_SIZE);
-    init_get_bits(&s->gb, t->bitstream_buf, 8*(bytestream2_get_bytes_left(&gb)));
+    s->dsp.bswap16_buf(s->bitstream_buf, (const uint16_t*)buf, (buf_end-buf)/2);
+    memset((uint8_t*)s->bitstream_buf + (buf_end-buf), 0, FF_INPUT_BUFFER_PADDING_SIZE);
+    init_get_bits(&s->gb, s->bitstream_buf, 8*(buf_end-buf));
+
     for (s->mb_y=0; s->mb_y < (avctx->height+15)/16; s->mb_y++)
         for (s->mb_x=0; s->mb_x < (avctx->width +15)/16; s->mb_x++)
-            if(decode_mb(t, inter) < 0)
+            if(decode_mb(s, inter) < 0)
                 return -1;
 
     *data_size = sizeof(AVFrame);
-    *(AVFrame*)data = t->frame;
+    *(AVFrame*)data = s->frame;
 
     if (chunk_type != MADe_TAG)
-        FFSWAP(AVFrame, t->frame, t->last_frame);
+        FFSWAP(AVFrame, s->frame, s->last_frame);
 
     return buf_size;
 }
@@ -335,11 +310,11 @@ static av_cold int decode_end(AVCodecContext *avctx)
 AVCodec ff_eamad_decoder = {
     .name           = "eamad",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_MAD,
+    .id             = AV_CODEC_ID_MAD,
     .priv_data_size = sizeof(MadContext),
     .init           = decode_init,
     .close          = decode_end,
     .decode         = decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("Electronic Arts Madcow Video")
+    .long_name      = NULL_IF_CONFIG_SMALL("Electronic Arts Madcow Video")
 };
