@@ -16,11 +16,15 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "config.h"
 #include "file.h"
 #include "log.h"
+#include "mem.h"
 #include <fcntl.h>
 #include <sys/stat.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #if HAVE_MMAP
 #include <sys/mman.h>
 #elif HAVE_MAPVIEWOFFILE
@@ -171,7 +175,6 @@ int av_tempfile(const char *prefix, char **filename, int log_offset, void *log_c
     if (fd < 0) {
         int err = AVERROR(errno);
         av_log(&file_log_ctx, AV_LOG_ERROR, "ff_tempfile: Cannot open temporary file %s\n", *filename);
-        av_freep(filename);
         return err;
     }
     return fd; /* success */
@@ -194,3 +197,4 @@ int main(void)
     return 0;
 }
 #endif
+
